@@ -18,6 +18,7 @@
  */
 
 #include "bamf-legacy-window-test.h"
+#include "bamf-legacy-screen-private.h"
 
 G_DEFINE_TYPE (BamfLegacyWindowTest, bamf_legacy_window_test, BAMF_TYPE_LEGACY_WINDOW);
 
@@ -126,16 +127,32 @@ bamf_legacy_window_test_is_skip_tasklist (BamfLegacyWindow *legacy_window)
 }
 
 void
-bamf_legacy_window_test_set_name (BamfLegacyWindowTest *self, char *val)
+bamf_legacy_window_test_set_name (BamfLegacyWindowTest *self, const char *val)
 {
   if (g_strcmp0 (self->name, val) == 0)
     return;
 
-  self->name = val;
+  self->name = g_strdup (val);
 
   g_signal_emit_by_name (self, "name-changed");
 }
 
+void
+bamf_legacy_window_test_set_wmclass (BamfLegacyWindowTest *self, const char *class_name, const char *instance_name)
+{
+  if (g_strcmp0 (self->wm_class_name, class_name) != 0)
+    {
+      g_free (self->wm_class_name);
+      self->wm_class_name = g_strdup (class_name);
+    }
+
+  if (g_strcmp0 (self->wm_class_instance, instance_name) != 0)
+    {
+      g_free (self->wm_class_instance);
+      self->wm_class_instance = g_strdup (instance_name);
+    }
+}
+
 static const char *
 bamf_legacy_window_test_get_name (BamfLegacyWindow *legacy_window)
 {
@@ -147,13 +164,23 @@ bamf_legacy_window_test_get_name (BamfLegacyWindow *legacy_window)
 }
 
 static const char *
-bamf_legacy_window_test_get_class (BamfLegacyWindow *legacy_window)
+bamf_legacy_window_test_get_class_name (BamfLegacyWindow *legacy_window)
 {
   BamfLegacyWindowTest *self;
 
   self = BAMF_LEGACY_WINDOW_TEST (legacy_window);
 
-  return  self->klass;
+  return self->wm_class_name;
+}
+
+static const char *
+bamf_legacy_window_test_get_class_instance_name (BamfLegacyWindow *legacy_window)
+{
+  BamfLegacyWindowTest *self;
+
+  self = BAMF_LEGACY_WINDOW_TEST (legacy_window);
+
+  return self->wm_class_instance;
 }
 
 char *
@@ -163,7 +190,7 @@ bamf_legacy_window_test_get_exec_string (BamfLegacyWindow *legacy_window)
 
   self = BAMF_LEGACY_WINDOW_TEST (legacy_window);
 
-  return self->exec;
+  return g_strdup (self->exec);
 }
 
 char *
@@ -174,7 +201,7 @@ bamf_legacy_window_test_get_app_id (BamfLegacyWindow *legacy_window)
 
   self = BAMF_LEGACY_WINDOW_TEST (legacy_window);
 
-  return self->application_id;
+  return g_strdup (self->application_id);
 }
 
 char *
@@ -185,7 +212,7 @@ bamf_legacy_window_test_get_unique_bus_name (BamfLegacyWindow *legacy_window)
 
   self = BAMF_LEGACY_WINDOW_TEST (legacy_window);
 
-  return self->unique_bus_name;
+  return g_strdup (self->unique_bus_name);
 }
 
 char *
@@ -196,7 +223,7 @@ bamf_legacy_window_test_get_menu_object_path (BamfLegacyWindow *legacy_window)
 
   self = BAMF_LEGACY_WINDOW_TEST (legacy_window);
 
-  return self->dbus_menu_object_path;
+  return g_strdup (self->dbus_menu_object_path);
 }
 
 void
@@ -229,6 +256,7 @@ bamf_legacy_window_test_maximized (BamfLegacyWindow *legacy_window)
 void
 bamf_legacy_window_test_close (BamfLegacyWindowTest *self)
 {
+  self->is_closed = TRUE;
   g_signal_emit_by_name (self, "closed");
 }
 
@@ -280,10 +308,75 @@ bamf_legacy_window_test_set_dbus_menu_object_path (BamfLegacyWindowTest *self, c
   self->dbus_menu_object_path = g_strdup (object_path);
 }
 
-void
-bamf_legacy_window_test_dispose (GObject *object)
+gboolean
+bamf_legacy_window_test_is_closed (BamfLegacyWindow *window)
 {
-  G_OBJECT_CLASS (bamf_legacy_window_test_parent_class)->dispose (object);
+  g_return_val_if_fail (BAMF_IS_LEGACY_WINDOW_TEST (window), TRUE);
+
+  BamfLegacyWindowTest *self = BAMF_LEGACY_WINDOW_TEST (window);
+  return self->is_closed;
+}
+
+static void
+handle_destroy_notify (BamfLegacyWindowTest *copy, BamfLegacyWindowTest *self_was_here)
+{
+  BamfLegacyScreen *screen = bamf_legacy_screen_get_default ();
+  _bamf_legacy_screen_open_test_window (screen, copy);
+}
+
+void
+bamf_legacy_window_test_reopen (BamfLegacyWindow *window)
+{
+  g_return_if_fail (BAMF_IS_LEGACY_WINDOW_TEST (window));
+
+  BamfLegacyWindowTest *self = BAMF_LEGACY_WINDOW_TEST (window);
+  BamfLegacyWindowTest *copy = bamf_legacy_window_copy (self);
+  g_object_weak_ref (G_OBJECT (self), (GWeakNotify) handle_destroy_notify, copy);
+  bamf_legacy_window_test_close (self);
+}
+
+BamfWindowType
+bamf_legacy_window_test_get_window_type (BamfLegacyWindow *window)
+{
+  g_return_val_if_fail (BAMF_IS_LEGACY_WINDOW_TEST (window), TRUE);
+
+  BamfLegacyWindowTest *self = BAMF_LEGACY_WINDOW_TEST (window);
+  return self->window_type;
+}
+
+char *
+bamf_legacy_window_test_get_hint (BamfLegacyWindow *window, const char *name)
+{
+  g_return_val_if_fail (BAMF_IS_LEGACY_WINDOW_TEST (window), NULL);
+  BamfLegacyWindowTest *self = BAMF_LEGACY_WINDOW_TEST (window);
+
+  return g_strdup (g_hash_table_lookup (self->hints, name));
+}
+
+void
+bamf_legacy_window_test_set_hint (BamfLegacyWindow *window, const char *name, const char *value)
+{
+  g_return_if_fail (BAMF_IS_LEGACY_WINDOW_TEST (window));
+  BamfLegacyWindowTest *self = BAMF_LEGACY_WINDOW_TEST (window);
+
+  return g_hash_table_insert (self->hints, g_strdup (name), g_strdup (value));
+}
+
+static void
+bamf_legacy_window_test_finalize (GObject *object)
+{
+  BamfLegacyWindowTest *self = BAMF_LEGACY_WINDOW_TEST (object);
+
+  g_free (self->name);
+  g_free (self->wm_class_name);
+  g_free (self->wm_class_instance);
+  g_free (self->exec);
+  g_free (self->application_id);
+  g_free (self->unique_bus_name);
+  g_free (self->dbus_menu_object_path);
+  g_hash_table_unref (self->hints);
+
+  G_OBJECT_CLASS (bamf_legacy_window_test_parent_class)->finalize (object);
 }
 
 void
@@ -292,9 +385,10 @@ bamf_legacy_window_test_class_init (BamfLegacyWindowTestClass *klass)
   BamfLegacyWindowClass *win_class = BAMF_LEGACY_WINDOW_CLASS (klass);
   GObjectClass *obj_class = G_OBJECT_CLASS (klass);
 
-  obj_class->dispose          = bamf_legacy_window_test_dispose;
+  obj_class->finalize         = bamf_legacy_window_test_finalize;
   win_class->get_name         = bamf_legacy_window_test_get_name;
-  win_class->get_class_name   = bamf_legacy_window_test_get_class;
+  win_class->get_class_name   = bamf_legacy_window_test_get_class_name;
+  win_class->get_class_instance_name = bamf_legacy_window_test_get_class_instance_name;
   win_class->get_exec_string  = bamf_legacy_window_test_get_exec_string;
   win_class->get_xid          = bamf_legacy_window_test_get_xid;
   win_class->get_pid          = bamf_legacy_window_test_get_pid;
@@ -306,7 +400,12 @@ bamf_legacy_window_test_class_init (BamfLegacyWindowTestClass *klass)
   win_class->get_unique_bus_name = bamf_legacy_window_test_get_unique_bus_name;
   win_class->get_menu_object_path = bamf_legacy_window_test_get_menu_object_path;
   win_class->get_geometry     = bamf_legacy_window_test_get_geometry;
+  win_class->get_window_type  = bamf_legacy_window_test_get_window_type;
   win_class->maximized        = bamf_legacy_window_test_maximized;
+  win_class->is_closed        = bamf_legacy_window_test_is_closed;
+  win_class->get_hint         = bamf_legacy_window_test_get_hint;
+  win_class->set_hint         = bamf_legacy_window_test_set_hint;
+  win_class->reopen           = bamf_legacy_window_test_reopen;
 }
 
 
@@ -315,18 +414,49 @@ bamf_legacy_window_test_init (BamfLegacyWindowTest *self)
 {
   self->pid = g_random_int_range (1, 100000);
   self->maximized = BAMF_WINDOW_FLOATING;
+  self->is_closed = FALSE;
+  self->hints = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 }
 
 
 BamfLegacyWindowTest *
-bamf_legacy_window_test_new (guint32 xid, gchar *name, gchar *klass, gchar *exec)
+bamf_legacy_window_copy (BamfLegacyWindowTest *self)
+{
+  BamfLegacyWindowTest *copy;
+
+  copy = g_object_new (BAMF_TYPE_LEGACY_WINDOW_TEST, NULL);
+  copy->xid = self->xid;
+  copy->pid = self->pid;
+  copy->name = g_strdup (self->name);
+  copy->wm_class_name = g_strdup (self->wm_class_name);
+  copy->wm_class_instance = g_strdup (self->wm_class_instance);
+  copy->exec = g_strdup (self->exec);
+  copy->application_id = g_strdup (self->application_id);
+  copy->unique_bus_name = g_strdup (self->unique_bus_name);
+  copy->dbus_menu_object_path = g_strdup (self->dbus_menu_object_path);
+  copy->needs_attention = self->needs_attention;
+  copy->is_desktop = self->is_desktop;
+  copy->is_skip = self->is_skip;
+  copy->is_active = self->is_active;
+  copy->is_closed = self->is_closed;
+  copy->geometry = self->geometry;
+  copy->maximized = self->maximized;
+  copy->window_type = self->window_type;
+  copy->hints = g_hash_table_ref (self->hints);
+
+  return copy;
+}
+
+BamfLegacyWindowTest *
+bamf_legacy_window_test_new (guint32 xid, const gchar *name, const gchar *wmclass_name, const gchar *exec)
 {
   BamfLegacyWindowTest *self;
 
   self = g_object_new (BAMF_TYPE_LEGACY_WINDOW_TEST, NULL);
+  self->window_type = BAMF_WINDOW_NORMAL;
   self->xid = xid;
   self->name = g_strdup (name);
-  self->klass = g_strdup (klass);
+  self->wm_class_name = g_strdup (wmclass_name);
   self->exec = g_strdup (exec);
 
   return self;
