@@ -28,7 +28,7 @@
 #include "bamf.h"
 
 void test_application_create_suite (GDBusConnection *connection);
-void test_matcher_create_suite (void);
+void test_matcher_create_suite (GDBusConnection *connection);
 void test_view_create_suite (GDBusConnection *connection);
 void test_window_create_suite (void);
 
@@ -39,7 +39,7 @@ on_bus_acquired (GDBusConnection *connection, const gchar *name, gpointer data)
 {
   GMainLoop *loop = data;
 
-  test_matcher_create_suite ();
+  test_matcher_create_suite (connection);
   test_view_create_suite (connection);
   test_window_create_suite ();
   test_application_create_suite (connection);
